==> xl-react-components-0.18.103.tgz: package/app/xl-react-form/xl-react-dip-redux-form/xl-react-dip-redux-form-reducer.js <==
import R from 'ramda';
import {actionReducer, composeReducer, stateIdentity} from '../../pages/reducers/reducer-utils';
import {prepareGroupedProperties} from '../xl-react-dip-dynamic-form-service';
import {getExternalPropertyKeys} from '../form-model-utils';
import formActions from './xl-react-dip-redux-form-actions';
import {createInitialModel} from './xl-react-dip-redux-form-model-utils';

const initializeForm = actionReducer((state, action) => ({
    ...state,
    [action.formId]: {
        ...R.omit(['rest', 'type'], action),
        ...state[action.formId],
        collapsed: action.rest.collapsed,
        groupedProperties: prepareGroupedProperties(action.rest.metadata, action.rest.collapsed),
        ignoredProperties: action.rest.ignoredProperties,
        metadata: action.rest.metadata,
        model: createInitialModel({
            ...action.rest,
            externalProperties: getExternalPropertyKeys(action.rest.model),
            ignoredProperties: action.rest.ignoredProperties || []
        }),
        pristine: action.rest.pristine
    }
}));

const mergeContext = actionReducer((state, action) => {
    const currentState = state[action.formId];

    if (currentState) {
        const mergedContext = R.mergeDeepRight(currentState, action.context);
        const groupedProperties = prepareGroupedProperties(mergedContext.metadata, mergedContext.collapsed);
        const {model} = action.context;
        const newContext =
            R.pipe(
                R.assoc('groupedProperties', groupedProperties),
                R.assoc('model', model ? model : currentState.model)
            )(mergedContext);

        if (action.context.model) {
            R.assoc('groupedProperties', groupedProperties, mergedContext);
        }
        return {
            ...state,
            [action.formId]: newContext
        };
    }

    return state;
});

const setContext = actionReducer((state, action) => {
    const currentState = state[action.formId];
    const {collapsed, metadata, model} = action.context;
    const groupedProperties = prepareGroupedProperties(metadata, collapsed);
    const newContext =
        R.pipe(
            R.assoc('groupedProperties', groupedProperties),
            R.assoc('model', model ? model : currentState.model)
        )(action.context);

    if (action.context.model) {
        R.assoc('groupedProperties', groupedProperties, action.context);
    }

    return state[action.formId] ? {
        ...state,
        [action.formId]: newContext
    } : state;
});

const openCategoryGroup = actionReducer((state, action) => {
    const transform = (item) => ({...item, isOpen: action.categoryName === item.category ? true : item.isOpen});

    return {
        ...state,
        [action.formId]: {
            ...state[action.formId],
            groupedProperties: R.map(transform, state[action.formId].groupedProperties)
        }
    };
});

const toggleCategoryGroup = actionReducer((state, action) => {
    const transform = (item) => ({...item, isOpen: action.categoryName === item.category ? !item.isOpen : item.isOpen});

    return {
        ...state,
        [action.formId]: {
            ...state[action.formId],
            groupedProperties: R.map(transform, state[action.formId].groupedProperties)
        }
    };
});

const unmount = actionReducer((state, action) => R.omit([action.formId], state));

const updateExternalProperties = actionReducer((state, action) => state[action.formId] ? {
    ...state,
    [action.formId]: {
        ...state[action.formId],
        model: {
            ...state[action.formId].model,
            externalProperties: action.externalProperties
        }
    }
} : state);

const allValuesAreTrue = R.pipe(R.values, R.all(R.equals(true)));

const changePristineState = actionReducer((state, action) => ({
    ...state,
    [action.formId]: {
        ...state[action.formId],
        pristine: allValuesAreTrue(action.pristinity),
        pristinity: action.pristinity
    }
}));

const changeValidity = actionReducer((state, action) => ({
    ...state,
    [action.formId]: {
        ...state[action.formId],
        valid: allValuesAreTrue(action.validity),
        validity: action.validity
    }
}));

export const xlReactDipReduxFormReducer = (initialState = {}) => composeReducer(
    initializeForm(formActions.INITIALIZE),
    mergeContext(formActions.MERGE_CONTEXT),
    openCategoryGroup(formActions.OPEN_CATEGORY_GROUP),
    setContext(formActions.SET_CONTEXT),
    toggleCategoryGroup(formActions.TOGGLE_CATEGORY_GROUP),
    unmount(formActions.UNMOUNT),
    updateExternalProperties(formActions.UPDATE_EXTERNAL_PROPERTIES),
    changePristineState(formActions.CHANGE_PRISTINE_STATE),
    changeValidity(formActions.CHANGE_VALIDITY),
    stateIdentity
)(initialState);

==> xl-react-components-0.21.15.tgz: package/app/xl-react-form/xl-react-dip-redux-form/xl-react-dip-redux-form-reducer.js <==
import R from 'ramda';
import RS from 'ramdasauce';
import RU from '../../utils/ramda-utils';
import {actionReducer, composeReducer, stateIdentity} from '../../pages/reducers/reducer-utils';
import {getInitialContext, prepareGroupedProperties, validateAllFields} from '../xl-react-dip-dynamic-form-service';
import {getExternalPropertyKeys} from '../form-model-utils';
import formActions from './xl-react-dip-redux-form-actions';
import {createInitialModel} from './xl-react-dip-redux-form-model-utils';

const pathToValidationMessages = ['model', 'validation-messages'];

const calculatePristinity = (model) => ({
    ...R.pipe(
        R.prop('external-properties'),
        R.keys,
        R.reduce((acc, val) => ({...acc, [val]: R.propOr(true, 'val', acc)}), {})
    )(model),
    ...R.pipe(
        R.keys,
        R.filter((key) => !R.equals(key, 'external-properties')),
        R.reduce((acc, val) => ({...acc, [val]: R.propOr(true, 'val', acc)}), {})
    )(model)
});

const initializeForm = actionReducer((state, action) => {
    const model = createInitialModel({
        ...action.rest,
        externalProperties: getExternalPropertyKeys(action.rest.model),
        ignoredProperties: R.pathOr([], ['rest', 'ignoredProperties'], action)
    });
    const context = getInitialContext(action.rest, model);
    const props = {
        ...R.omit(['rest', 'type'], action),
        ...state[action.formId],
        collapsed: action.rest.collapsed,
        context,
        groupedProperties: prepareGroupedProperties(action.rest.metadata, action.rest.collapsed),
        handlers: R.pathOr({}, ['rest', 'handlers'], action),
        ignoredProperties: action.rest.ignoredProperties,
        message: R.pathOr({show: false}, ['rest', 'message'], action),
        metadata: action.rest.metadata,
        model,
        objLabel: R.pathOr('id', ['rest', 'objLabel'], action),
        pristinity: calculatePristinity(model),
        validators: R.pathOr([], ['rest', 'validators'], action)
    };

    const {validity, messages} = validateAllFields(props);

    return {
        ...state,
        [action.formId]: {
            ...props,
            messages,
            validity
        }
    };
});

const mergeContext = actionReducer((state, action) => {
    const currentState = state[action.formId];

    if (currentState) {
        const mergedContext = R.mergeDeepRight(currentState, action.context);
        const groupedProperties = prepareGroupedProperties(mergedContext.metadata, mergedContext.collapsed);
        const model = R.pipe(
            R.keys,
            R.reduce((acc, key) => R.assoc(key, R.path(['context', 'model', key], action), acc), currentState.model)
        )(action.context.model);
        const newContext =
            R.pipe(
                R.assoc('groupedProperties', groupedProperties),
                R.assoc('model', RS.isNilOrEmpty(model) ? currentState.model : model),
                RU.removeEmptyProperty(pathToValidationMessages)
            )(mergedContext);

        return {
            ...state,
            [action.formId]: newContext
        };
    }

    return state;
});

const removeValidationMessages = actionReducer((state, action) => {
    const currentState = state[action.formId];
    if (currentState) {
        const validationMessages = R.pathOr([], pathToValidationMessages, currentState);
        const filteredValidationMessages = R.filter(
            (message) => !R.contains(message.property, action.propertyNames),
            validationMessages);
        return {
            ...state,
            [action.formId]: R.pipe(
                R.assocPath(pathToValidationMessages, filteredValidationMessages),
                RU.removeEmptyProperty(pathToValidationMessages)
            )(currentState)
        };
    }
    return state;
});

const setContext = actionReducer((state, action) => {
    const currentState = state[action.formId];
    const {collapsed, metadata, model} = action.context;
    const groupedProperties = prepareGroupedProperties(metadata, collapsed);
    const newContext =
        R.pipe(
            R.assoc('groupedProperties', groupedProperties),
            R.assoc('model', model ? model : currentState.model)
        )(action.context);

    if (action.context.model) {
        R.assoc('groupedProperties', groupedProperties, action.context);
    }

    return state[action.formId] ? {
        ...state,
        [action.formId]: newContext
    } : state;
});

const openCategoryGroup = actionReducer((state, action) => {
    const transform = (item) => ({...item, isOpen: action.categoryName === item.category ? true : item.isOpen});

    return {
        ...state,
        [action.formId]: {
            ...state[action.formId],
            groupedProperties: R.map(transform, state[action.formId].groupedProperties)
        }
    };
});

const toggleCategoryGroup = actionReducer((state, action) => {
    const transform = (item) => ({...item, isOpen: action.categoryName === item.category ? !item.isOpen : item.isOpen});

    return {
        ...state,
        [action.formId]: {
            ...state[action.formId],
            groupedProperties: R.map(transform, state[action.formId].groupedProperties)
        }
    };
});

const unmount = actionReducer((state, action) => R.omit([action.formId], state));

const updateExternalProperties = actionReducer((state, action) => state[action.formId] ? {
    ...state,
    [action.formId]: {
        ...state[action.formId],
        model: action.model
    }
} : state);

const changePristineState = actionReducer((state, action) => ({
    ...state,
    [action.formId]: {
        ...state[action.formId],
        pristinity: action.pristinity
    }
}));

const changeValidity = actionReducer((state, action) => ({
    ...state,
    [action.formId]: {
        ...state[action.formId],
        validity: action.validity
    }
}));

export const xlReactDipReduxFormReducer = (initialState = {}) => composeReducer(
    initializeForm(formActions.INITIALIZE),
    mergeContext(formActions.MERGE_CONTEXT),
    openCategoryGroup(formActions.OPEN_CATEGORY_GROUP),
    removeValidationMessages(formActions.REMOVE_VALIDATION_MESSAGES),
    setContext(formActions.SET_CONTEXT),
    toggleCategoryGroup(formActions.TOGGLE_CATEGORY_GROUP),
    unmount(formActions.UNMOUNT),
    updateExternalProperties(formActions.UPDATE_EXTERNAL_PROPERTIES),
    changePristineState(formActions.CHANGE_PRISTINE_STATE),
    changeValidity(formActions.CHANGE_VALIDITY),
    stateIdentity
)(initialState);
